function [logc_obs,age,logC,Rc,Rd,Rf,PCratio,PDratio,alpha,X] ...
    = cCAPM_Sim3(I,T,burnin,beta,gamma,delta,g,A,Sigma,sigma_i,sigma_0,sigma_e,N,dividendLeverage)
% simulates heterogeneous-agent cCAPM model that is analytically solvable
% I: number of agents
% T: time periods
% burnin: number of periods as burn in
% beta: discount factor
% gamma: relative risk aversion
% delta: death probability
% g: average log consumption growth and log dividend growth
% A: coefficient matrix of VAR
% Sigma: variance-covariance matrix of log consumption growth and log
% dividend growth
% sigma_i: idiosyncratic volatility
% sigma_0: standard deviation at birth
% sigma_e: measurement error
% N: truncation parameter for computing asset pricing model
% dividendLeverage: dividend leverage

if nargin < 14
    dividendLeverage = 1;
end

if dividendLeverage <= 0
    error('dividendLeverage must be positive')
end

% solve asset pricing model
if max(abs(eig(A)))>=1
    error('Process is nonstationary')
end
P = (eye(2)-A)\(Sigma/(eye(2)-A')); % Sigma tilde in paper
vc = [1-gamma,0]'; % vector to compute consumption claim
vd = [-gamma,dividendLeverage]'; % vector to compute dividend claim
rc = beta*(1-delta)*exp(1/2*gamma*(gamma+1)*sigma_i^2+vc'*g+1/2*vc'*P*vc);
rd = beta*(1-delta)*exp(1/2*gamma*(gamma+1)*sigma_i^2+vd'*g+1/2*vd'*P*vd);
% the number in Eq (3.6) of paper
r = max([rc rd]);
if r>=1
    r
    error('Equilibrium does not exist')
end

ac = zeros(1,N);
bc = zeros(2,N);
ad = zeros(1,N);
bd = zeros(2,N);

Sigman=zeros(2);
Bn=zeros(2);
for n=1:N
    Sigman=A*(Sigman+P)*A'; % Sigma_n on p. 7 of paper
    Bn=A*(eye(2)+Bn); % B_n on p. 7 of paper
    temp=Bn*P;
    Dn=-temp-temp'+Sigman; % Omega_n on p. 7 of paper
    ac(n)=1/2*vc'*Dn*vc;
    bc(:,n)=Bn'*vc;
    ad(n)=1/2*vd'*Dn*vd;
    bd(:,n)=Bn'*vd;
end
coeffc = rc.^[1:N];
coeffd = rd.^[1:N];
Binf=A/(eye(2)-A); % limit of B_n
D=-Binf*P-P*Binf'+Sigman; % limit of Omega_n
Vc =@(x)(sum(coeffc.*exp(ac+(x-g)'*bc))+(rc^(N+1)/(1-rc))*...
    (vc'*Binf*(x-g)+1/2*vc'*D*vc));
Vd =@(x)(sum(coeffd.*exp(ad+(x-g)'*bd))+(rd^(N+1)/(1-rd))*...
    (vd'*Binf*(x-g)+1/2*vd'*D*vd));
% price-consumption and price-dividend ratio computed by replacing the terms beyond N by their
% limit to calculate the infinite sum explicitly; see Eq (3.5) in paper

% generate aggregate shocks, compute aggregate and individual consumption
% and age, and price-dividend ratio
b = burnin; % number of burn-in periods
bT = b+T; % total number of periods to simulate
MU=zeros(bT,2);
U=(mvnrnd(MU,Sigma))'; % generate 2 x bT aggregate shocks (error term of VAR)
x=g; % set initial value to unconditional mean
X=zeros(2,bT); % 2 x bT matrix of log consumption/dividend growth
X(:,1)=x; % set initial value
PCratio=zeros(1,bT); % realization of price-consumption ratio
PDratio=zeros(1,bT); % realization of price-dividend ratio
PCratio(1)=Vc(x); % set initial value
PDratio(1)=Vd(x); % set initial value
logC=zeros(1,bT); % log aggregate consumption
death = binornd(1,delta,[I,bT]); % indicator of death
idshock = normrnd(-sigma_i^2/2,sigma_i,[I,bT]); % idiosyncratic shock between periods
birthshock = normrnd(-sigma_0^2/2,sigma_0,[I,bT]); % idiosyncratic shock at birth
logci=zeros(I,bT); % log individual consumption
age=zeros(I,bT); % age
logci(:,1)=birthshock(:,1); % initial log individual consumption
age(:,1)=0;
for t=2:bT
    x=g+A*(x-g)+U(:,t); % update log consumption/dividend growth
    X(:,t)=x; % current log consumption/dividend growth
    PCratio(t)=Vc(x); % compute current price-consumption ratio
    PDratio(t)=Vd(x); % compute current price-dividend ratio
    logC(t)=logC(t-1)+x(1); % compute aggregate consumption
    % x(1)=log(C_t/C_{t-1}), so log C_t = log C_{t-1}+x(1)
    logci(:,t)=(1-death(:,t)).*(logci(:,t-1)+x(1)+idshock(:,t))...
        +death(:,t).*(logC(t)+birthshock(:,t)); % individual consumption
    % log individual consumption grows by aggregate consumption plus
    % idiosyncratic shock if alive; it is reset to aggregate consumption
    % plus some idiosyncratic shock if reborn
    age(:,t)=(1-death(:,t)).*(age(:,t-1)+1); % age is +1 if alive, otherwise 0
end
merror = normrnd(-sigma_e^2/2,sigma_e,[I,bT]); % measurement error
logc_obs=logci+merror; % observed log consumption

% compute asset returns
tempC = [Vc(g),PCratio]; % PC ratio from t=0 to bT
tempD = [Vd(g),PDratio]; % PD ratio from t=0 to bT
Rc = (tempC(2:end)+1)./tempC(1:end-1).*exp(X(1,:)); % consumption claim return from t-1 to t; see Eq (3.7) in paper
Rd = (tempD(2:end)+1)./tempD(1:end-1).*exp(dividendLeverage*X(2,:)); % dividend claim return from t-1 to t; see Eq (3.7) in paper
vf = [-gamma,0]';
Rf = 1./(beta*(1-delta)*exp(1/2*gamma*(gamma+1)*sigma_i^2+vf'*((eye(2)-A)*g*ones(1,bT)+A*X)+1/2*vf'*Sigma*vf));
% risk-free rate from t to t+1; see Eq (3.8) in paper

% delete burn in
logc_obs(:,1:b)=[];
age(:,1:b)=[];
logC(1:b)=[];
Rc(1:b)=[];
Rd(1:b)=[];
Rf(1:b)=[];
PCratio(1:b)=[];
PDratio(1:b)=[];
X(:,1:b)=[];

% power law exponent
alpha1 = (1+sqrt(1+8*delta/sigma_i^2))/2;
alpha2 = alpha1-1;
alpha = [alpha1 alpha2];

end